#ifndef IWIDGETDEVICE_H
#define IWIDGETDEVICE_H

#include <QObject>
#include <QVector>
#include <QMap>

#define DEVICE_ID_PCIE_A708             0x9480      ///< идентификатор поддерживаемого устройства A708 PCIe
#define DEVICE_ID_MPCIE_A708            0x9479      ///< идентификатор поддерживаемого устройства A708 mPCIe


#define DEVICE_REV_ID_A708_M_UD2        0x11        ///< ревизия поддерживаемого устройства A708 mPCIe UD2
#define DEVICE_REV_ID_A708_UD2          0x01        ///< ревизия поддерживаемого устройства A708 PCIe UD2


///
/// \brief The DeviceStatistics struct наработанная статистика платы, набранная за прохождение электропрогона
///
struct DeviceStatistics {
    QVector<int> mTMsg;             ///< сообщения передатчиков
    QVector<int> mTErr;             ///< ошибки передатчиков
    QVector<int> mRMsg;             ///< сообщения приёмников
    QVector<int> mRErr;             ///< ошибки приёмников
    QVector<int> mRkOut;            ///< выходные разовые команды
    QVector<int> mRkIn;             ///< входные разовые команды
    QVector<int> mA708T;
    QVector<int> mA708R;
    QVector<int> mA708Err;
};

///
/// \brief The DeviceStatisticsParametres struct параметры наработанной статитсики для платы
///
struct DeviceStatisticsParametres {
    int mCountT;        ///< кол-во передатчиков
    int mCountR;        ///< кол-во приёмников
    int mCountRkIn;     ///< кол-во вх рк
    int mCountRkOut;    ///< кол-во вых рк
    DeviceStatisticsParametres(){}
    ///
    /// \brief DeviceStatisticsParametres конструктор с параметрами
    /// \param countT Кол-во передатчиков
    /// \param countR Кол-во приёмников
    /// \param countRkIn Кол-во входных РК
    /// \param countRkOut Кол-во выходных РК
    ///
    DeviceStatisticsParametres(int countT, int countR, int countRkIn, int countRkOut):
    mCountT(countT), mCountR(countR), mCountRkIn(countRkIn),mCountRkOut(countRkOut){}
};

///
/// \brief The ElectroprogonParametres struct параметры настроек плат для электропрогона
///
struct ElectroprogonParametres {
   int mSpeed;      ///< скорость 0 - 100; 1 - 12-14; 2 - 50
   bool mParity;    ///< бит чётности
   bool mReverce;   ///< реверс
   bool mOnRRT;     ///< включение RRT
   int mDiscrRRT;   ///< дискретность RRT 0 - 1; 1 - 10
   int mValueRRT;   ///< значение RRT
   int mNumA708T;   ///< номер передатчика А708 1 или 2
   ElectroprogonParametres():mNumA708T(1){}
   ///
   /// \brief ElectroprogonParametres Конструктор с параметрами
   /// \param speed скорость
   /// \param parity бит паритета
   /// \param reverce нумерация бит лейбла
   /// \param onRRT использование RRT
   /// \param discrRRT дискретность RRT
   /// \param valRRT значение RRT
   ///
   ElectroprogonParametres(int speed, bool parity, bool reverce, bool onRRT, int discrRRT, int valRRT, int numA708T):
   mSpeed(speed), mParity(parity), mReverce(reverce), mOnRRT(onRRT), mDiscrRRT(discrRRT), mValueRRT(valRRT), mNumA708T(numA708T){}
};

///
/// \brief The IWidgetDevice class интерфейс виджета, описывающий статистику платы
///
class IWidgetDevice {
public:
    IWidgetDevice() {
       mDeviceStatisticsParametresMap = generateDeviceStatisticsParametresMap();
    }
    ///
    /// \brief setDeviceName установить имя девайса
    /// \param name имя девайса
    ///
    virtual void setDeviceName(const QString& name) = 0;
    ///
    /// \brief getDeviceName получить имя девайса
    /// \return имя девайса
    ///
    virtual QString getDeviceName() = 0;
    ///
    /// \brief setDeviceId установить тип платы
    /// \param id тип платы
    ///
    virtual void setDeviceId(int id) = 0;
    ///
    /// \brief setDeviceStatistics установить наработанную статистику платы
    /// \param devStat наработанная статистика
    ///
    virtual void setDeviceStatistics(const DeviceStatistics& devStat) = 0;
    ///
    /// \brief getStringStatistics получить статистику в текстовом виде
    /// \return статистика в текстовом аиде
    ///
    virtual QStringList getStringStatistics() = 0;
protected:
    QMap<int, DeviceStatisticsParametres> mDeviceStatisticsParametresMap;   ///< контрольные параметры статистики по платам
    DeviceStatistics mCurrentStatictics;                                    ///< текущая статистика
public:
    ///
    /// \brief generateDeviceStatisticsParametresMap генерация контрольных параметров статистики по платам
    /// \return контрольные параметры статистики по платам
    ///
    static QMap<int, DeviceStatisticsParametres> generateDeviceStatisticsParametresMap() {
       QMap<int, DeviceStatisticsParametres> deviceStatisticsParametresMap;
       deviceStatisticsParametresMap.insert(DEVICE_ID_MPCIE_A708, DeviceStatisticsParametres(2,4,1,1));
       deviceStatisticsParametresMap.insert(DEVICE_ID_PCIE_A708, DeviceStatisticsParametres(2,4,0,0));
       return deviceStatisticsParametresMap;
    }
    ///
    /// \brief generateDevStat генерация нулевой статистики
    /// \param devStatParam контрольные параметры статистики
    /// \return нулевая статистика
    ///
    static DeviceStatistics generateDevStat(DeviceStatisticsParametres devStatParam)
    {
        DeviceStatistics devStat;
        for (int index=0; index < devStatParam.mCountT; index++) {
            devStat.mTMsg.append(0);
            devStat.mTErr.append(0);
        }
        for (int index=0; index < devStatParam.mCountR; index++) {
            devStat.mRMsg.append(0);
            devStat.mRErr.append(0);
        }
        for (int index=0; index < devStatParam.mCountRkIn; index++) {
            devStat.mRkIn.append(0);
        }
        for (int index=0; index < devStatParam.mCountRkOut; index++) {
            devStat.mRkOut.append(0);
        }
        for (int index=0; index < 2; index++) {
            devStat.mA708Err.append(0);
            devStat.mA708R.append(0);
            devStat.mA708T.append(0);
        }
        return devStat;
    }
};

#endif // IWIDGETDEVICE_H
