#include <QSpacerItem>
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "tools/channelfinder.h"
#include "a708_library.h"
#include "version.h"
#include <QMessageBox>
#include "dialoghtml.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    this->setWindowTitle(QString("%1 - версия %2")
                         .arg("A708. Электропрогон")
                         .arg(VERSION_APP)
                         );
    connect(this, &MainWindow::signalOnCreate, &MainWindow::onCreate);
    connect(&mTimerTracking, &QTimer::timeout, this, &MainWindow::onTimerTracking);
    connect(ui->pushButtonStart, &QPushButton::clicked, this, &MainWindow::onStart);
    connect(ui->pushButtonStop, &QPushButton::clicked, this, &MainWindow::onStop);
    connect(ui->pushButtonTerminalClear, &QPushButton::clicked, ui->listWidgetTerminal, &QListWidget::clear);
    connect(ui->pushButtonHtmlLog, &QPushButton::clicked, this, &MainWindow::onShowHtmlLog);
    connect(this, &MainWindow::signalPrint, &MainWindow::onPrint);
    connect(this, &MainWindow::signalPrintLines, &MainWindow::onPrintLines);
    emit signalOnCreate();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::onPrint(QString line)
{
    QString appendedLine = QString("%1: %2")
            .arg(QDateTime::currentDateTime().toString("yyyy.MM.dd hh:mm:ss"))
            .arg(line);
    ui->listWidgetTerminal->addItem(appendedLine);
    mLogger.appendText(appendedLine);
    ui->listWidgetTerminal->setCurrentRow(ui->listWidgetTerminal->count()-1);
}

void MainWindow::onPrintLines(QStringList lines)
{
    QString appendedLine = QString("%1:")
            .arg(QDateTime::currentDateTime().toString("yyyy.MM.dd hh:mm:ss"));
    ui->listWidgetTerminal->addItem(appendedLine);
    foreach (QString line, lines) {
        ui->listWidgetTerminal->addItem(line);
        mLogger.appendText(line);
    }
    ui->listWidgetTerminal->setCurrentRow(ui->listWidgetTerminal->count()-1);
}

void MainWindow::onCreate()
{
    mNumA708T = 1;
    mLogger.setPrefixName(QString("a708elprogon"));
    registerUserTypes();
    initComboBoxTimeoutElectroprogon();
    initDeviceList();
    connect(ui->checkBoxRrt, &QCheckBox::toggled, this, [this](bool checked)->void {
        ui->frameRrt->setEnabled(checked);
    });
    auto keys = mDevices.keys();
    foreach (int key, keys) {
        int fd = a708_open(mDevices.value(key).toLatin1().constData());
        VERSION verInfo;
        a708_getDeviceInfo(fd, &verInfo);
        a708_close(fd);
        mTasks.insert(key, new OneDeviceElectroprogonTask(verInfo.device_id, key, mDeviceInfoItems.value(key)));
    }
    auto keysTasks = mTasks.keys();
    foreach (int key, keysTasks) {
        connect(this, &MainWindow::signalElectroprogonParametresSet, mTasks.operator [](key), &OneDeviceElectroprogonTask::onElectroprogonParametresSet);
        connect(this, &MainWindow::signalStartTask, mTasks.operator [](key), &OneDeviceElectroprogonTask::onStart);
        connect(this, &MainWindow::signalStopTask, mTasks.operator [](key), &OneDeviceElectroprogonTask::onStop);
        connect(mTasks.value(key), &OneDeviceElectroprogonTask::signalHasFinished, this, &MainWindow::onFinished);
        connect(mTasks.value(key), &OneDeviceElectroprogonTask::signalSendStatistics, this, &MainWindow::onReceiveStatistics);
        connect(mTasks.value(key), &OneDeviceElectroprogonTask::signalPrint, this, &MainWindow::onPrint);
        connect(mTasks.value(key), &OneDeviceElectroprogonTask::signalPrintLines, this, &MainWindow::onPrintLines);
    }
}

QString MainWindow::timeFromMsToString(quint64 time_ms)
{
    auto ms = time_ms % 1000;
    auto sec = time_ms / 1000;
    auto min = sec / 60;
    auto hour = min / 60;
    min = min % 60;
    sec = sec - (hour * 60 * 60 + min * 60);
    return QString("%1 : %2 : %3")
            .arg(hour)
            .arg(QString::number(min).rightJustified(2, '0'))
            .arg(QString::number(sec).rightJustified(2, '0'))
            ;
}

void MainWindow::registerUserTypes()
{
    qRegisterMetaType<DeviceStatistics>("DeviceStatistics");
    qRegisterMetaType<DeviceStatisticsParametres>("DeviceStatisticsParametres");
    qRegisterMetaType<ElectroprogonParametres>("ElectroprogonParametres");
}

void MainWindow::onTimerTracking()
{
    mCurrentTimeTracking += cStepTracking;
    // печать времени на экран
    ui->labelTimerValue->setText(timeFromMsToString(mMaxTimeout*cStepTracking - mCurrentTimeTracking));

    if (mCurrentTimeTracking >= mMaxTimeout*cStepTracking) {
        if (ui->comboBoxSpeed->currentIndex() == ui->comboBoxSpeed->count()-1){
            mIsAutoMode = false;
        } else {
            if (!ui->checkBoxRrt->isChecked()) // режим - авто
                mIsAutoMode = true;
            else
                mIsAutoMode = false;
        }
        ui->pushButtonStop->click();
    }
}

QVector<MemElement> memProgon;

void MainWindow::onStart()
{
    if (mIsAutoMode) { // режим - авто
        mLogger.resumeLog();
    } else {
        mLogger.startLog();
    }

    QString memName = QString("%1/mem/%2").arg(QApplication::applicationDirPath()).arg("desk_test_progon.mem");
    auto isReaded = MemLoader::loadMemFile(memName, memProgon);
    if (!isReaded) {
        emit signalPrint(QString("не удалось считать mem файл: '%1'")
                         .arg(memName)
                         );
    }
    mIsAutoMode = false;
    ui->pushButtonStart->setEnabled(false);
    ui->pushButtonStop->setEnabled(true);
    ui->comboBoxTimeoutElectroprogon->setEnabled(false);
    ui->tabParams->setEnabled(false);
    mCurrentTimeTracking = 0;
    mCountFinished = 0;
    mMaxTimeout = ui->comboBoxTimeoutElectroprogon->currentData().toInt() * 60;
    ElectroprogonParametres params = ElectroprogonParametres(
                ui->comboBoxSpeed->currentIndex(), true, false,
                ui->checkBoxRrt->isChecked(), ui->comboBoxRrtDiscrValue->currentIndex(), ui->spinBoxRrtValue->value(),
                mNumA708T
                );

    emit signalElectroprogonParametresSet(params);

    QTimer tmrDelay;
    QEventLoop loopDelay;
    connect(&tmrDelay, &QTimer::timeout, this, [&tmrDelay, &loopDelay, this](void)->void {
        if (loopDelay.isRunning() == false) return;
        tmrDelay.stop();
        loopDelay.exit();
    });
    tmrDelay.start(1000);
    loopDelay.exec();

    mTimerTracking.start(static_cast<int>(cStepTracking));
    // старт задач

    emit signalPrint(QString("%1 - версия %2")
                     .arg("a708. Электропрогон")
                     .arg(VERSION_APP));
    emit signalStartTask();
    emit signalPrint(QString("Старт электропрогона на время %1").arg(ui->comboBoxTimeoutElectroprogon->currentText()));
    emit signalPrint(QString("С параметрами:"));
    emit signalPrint(QString("скорость - %1, бит чётности - %2, реверс - %3")
                     .arg(ui->comboBoxSpeed->currentText())
                     .arg( true ? "да":"нет")
                     .arg( false ? "да":"нет")
                     );
    emit signalPrint(QString("использование RRT - %1, дискретность RRT - %2, значение RRT - %3 ")
                     .arg( ui->checkBoxRrt->isChecked() ? "да":"нет")
                     .arg(ui->comboBoxRrtDiscrValue->currentText())
                     .arg(QString::number(ui->spinBoxRrtValue->value()))
                     );
    emit signalPrint(QString("A708 ПЕРЕДАТЧИК = КАНАЛ %1")
                     .arg(QString::number(mNumA708T)));
    emit signalPrint(QString("для:"));
    auto keys = mDevices.keys();
    foreach (int key, keys) {
        emit signalPrint(mDeviceWidgets.value(key)->getDeviceName());
    }
    QString html;
    html += QString("<font size=\"3\" face=\"Liberation Mono\">");
    html += QString("<p>");
    html += QString("%1 - версия %2")
            .arg("a708. Электропрогон")
            .arg(VERSION_APP);
    html += QString("</p>");
    html += QString("<p>");
    html += QString("Старт электропрогона на время %1 с параметрами:").arg(ui->comboBoxTimeoutElectroprogon->currentText());
    html += QString("</p>");
    html += QString("<p>");
    html += QString("скорость - %1, бит чётности - %2, реверс - %3")
            .arg(ui->comboBoxSpeed->currentText())
            .arg( true ? "да":"нет")
            .arg( false ? "да":"нет");
    html += QString("</p>");
    html += QString("<p>");
    html += QString("использование RRT - %1, дискретность RRT - %2, значение RRT - %3 ")
            .arg( ui->checkBoxRrt->isChecked() ? "да":"нет")
            .arg(ui->comboBoxRrtDiscrValue->currentText())
            .arg(QString::number(ui->spinBoxRrtValue->value()));
    html += QString("</p>");
    html += QString("<p>");
    html += QString("A708 ПЕРЕДАТЧИК = КАНАЛ %1")
            .arg(QString::number(mNumA708T));
    html += QString("</p>");
    html += QString("</font>");
    mLogger.appendTextHtml(html);
}

void MainWindow::onStop()
{
    mTimerTracking.stop();
    emit signalStopTask();
    onPrint(QString("Останов электропрогона"));
    if (mTasks.count() == 0)
        onFinished();
}

void MainWindow::onShowHtmlLog()
{
    DialogHtml dialog(this);
    dialog.setModal(true);
    QFile file(mLogger.getPrevFileHtml());
    if (file.open(QIODevice::ReadOnly)) {
        auto html = QString::fromLocal8Bit(file.readAll());
        dialog.setHtml(html);
        dialog.exec();
        file.close();
    } else {
        QMessageBox::warning(this, "Внимание!", "Файл HTML-лога не найден!");
    }
}

void MainWindow::onFinished()
{
    mCountFinished++;
    if (mCountFinished >= mTasks.count()) {
        ui->pushButtonStart->setEnabled(true);
        ui->pushButtonStop->setEnabled(false);
        ui->comboBoxTimeoutElectroprogon->setEnabled(true);
        ui->tabParams->setEnabled(true);

        QTimer tmrDelay;
        QEventLoop loopDelay;
        connect(&tmrDelay, &QTimer::timeout, this, [&tmrDelay, &loopDelay](void)->void {
            if (loopDelay.isRunning() == false) return;
             tmrDelay.stop();
             loopDelay.exit();
        });
        tmrDelay.start(1000);
        loopDelay.exec();
        // печать итога прогона
        auto keys = mDevices.keys();
        for (int index = 0; index < keys.count(); index++) {
            auto resultStatLines = mDeviceWidgets.value(keys[index])->getStringStatistics();
            onPrintLines(resultStatLines);
            mLogger.appendTextHtml(mDeviceWidgets.value(keys[index])->getHtmlText());
        }
        mLogger.stopLog();
        if (mIsAutoMode) { // режим - авто
            mNumA708T++;
            // переключаем скорость
            ui->comboBoxSpeed->setCurrentIndex(ui->comboBoxSpeed->count()-1);
            // стартуем
            ui->pushButtonStart->click();
        } else {
            ui->comboBoxSpeed->setCurrentIndex(0);
            mNumA708T = 1;
            ui->pushButtonHtmlLog->setEnabled(true);
        }
    }
}

void MainWindow::onReceiveStatistics(int uid, DeviceStatistics devStat)
{
    mDeviceWidgets.value(uid)->setDeviceStatistics(devStat);
}

void MainWindow::onPressKeyBlinked(int key)
{
    int countBlinks = 0; // накопленное кол-во миганий
    QEventLoop loop;
    QTimer timer;
    // мигание лампочкой 3 раза
    connect(&timer, &QTimer::timeout, this, [this, &key, &countBlinks, &loop, &timer](void)->void {
        auto makeOneBlink = [this](int key)->void {
            int fd = a708_open(mDeviceInfoItems[key].mDevName.toLocal8Bit().constData());
            SADDR_DATA regData;
            regData.daddr = 0x1014;
            regData.data = 0x4;
            a708_writeReg(fd, &regData);
            a708_close(fd);
        };
        if (countBlinks++ == 3) {
            timer.stop();
            loop.exit();
            return;
        }
        makeOneBlink(key);
    });
    timer.start(1000); // мигать каждую секунду
    loop.exec(); // ожидаем пока отработает алгоритм миганий
}

void MainWindow::initComboBoxTimeoutElectroprogon()
{
    ui->comboBoxTimeoutElectroprogon->addItem("1мин", 1);
    ui->comboBoxTimeoutElectroprogon->addItem("15мин", 15);
    ui->comboBoxTimeoutElectroprogon->addItem("30мин", 30);
    ui->comboBoxTimeoutElectroprogon->addItem("45мин", 45);
    ui->comboBoxTimeoutElectroprogon->addItem("60мин", 60);
    ui->comboBoxTimeoutElectroprogon->addItem("1ч 30мин", 90);
    ui->comboBoxTimeoutElectroprogon->addItem("2ч 00мин", 120);
    ui->comboBoxTimeoutElectroprogon->addItem("2ч 30мин", 150);
    ui->comboBoxTimeoutElectroprogon->addItem("3ч 00мин", 180);
    ui->comboBoxTimeoutElectroprogon->addItem("3ч 30мин", 210);
    ui->comboBoxTimeoutElectroprogon->addItem("4ч 00мин", 240);
    for (int i=5; i<=48; i++){
        ui->comboBoxTimeoutElectroprogon->addItem(QString("%1ч 00мин").arg(QString::number(i)), i*60);
    }
}

void MainWindow::initDeviceList()
{
    auto devStatParams = IWidgetDevice::generateDeviceStatisticsParametresMap();
    auto devCount = ChannelFinder::getCountDevices();
    if (devCount == 0)
        return;
    auto devNames = ChannelFinder::getDeviceNameList();
    auto devInfoItems = ChannelFinder::getDeviceInfoList();
    for (int index = 0; index < devCount; index++) {
        mDeviceInfoItems.insert(index, devInfoItems[index]);
        int fd = a708_open(devNames.at(index).toLatin1().constData());
        VERSION verInfo;
        a708_getDeviceInfo(fd, &verInfo);
        a708_close(fd);
        // добавление платы
        mDevices.insert(index, devNames.at(index));
        WidgetDevice* widget = new WidgetDevice(ui->scrollAreaDevices);
        connect(widget, &WidgetDevice::pressedKeyBlinked, this, &MainWindow::onPressKeyBlinked);
        mDeviceWidgets.insert(index, widget);
        widget->setKey(index);
        widget->setDeviceId(verInfo.device_id);
        widget->setDeviceStatistics(IWidgetDevice::generateDevStat(devStatParams.value(verInfo.device_id)));
        widget->setBackClr( ((index % 2)==0) ? WidgetDevice::BACKCLR::light : WidgetDevice::BACKCLR::light);
        // составление имени устройства
        QString devName = devNames.at(index);
        QString devTypeStr;
        QString revIdStr;
        switch (verInfo.device_id) {
            case DEVICE_ID_MPCIE_A708: devTypeStr = tr("mPCIe");  break;
            case DEVICE_ID_PCIE_A708:  devTypeStr = tr("PCIe"); break;
        }
        switch (verInfo.revision) {
            case DEVICE_REV_ID_A708_M_UD2:
            case DEVICE_REV_ID_A708_UD2: revIdStr = tr("A429T=2, A429R=4, A708=2"); break;
        }
        widget->setDeviceName(QString("'%1'; DevID= 0x%2 (%3); RevID= 0x%4 (%5);")
                              .arg(devName)
                              .arg(QString::number(verInfo.device_id, 16))
                              .arg(devTypeStr)
                              .arg(QString::number(verInfo.revision, 16))
                              .arg(revIdStr)
                              );
        ui->scrollAreaWidgetContents->layout()->addWidget(widget);
    }
    /* Устанавливаем параметры растяжения областей.
     * Области считаются слева направо.
     * в Методе setStretchFactor вначале устанавливается индекс
     * области, а потом её вес.
     * Таким образом первой области ставим максимальный вес,
     * а второй области минимальный.
     * */
    ui->splitterDevTerm->setStretchFactor(0, 1);
    ui->splitterDevTerm->setStretchFactor(1, 0);
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    // очистка задач
    auto keysTasks = mTasks.keys();
    foreach (int key, keysTasks) {
        disconnect(this, &MainWindow::signalElectroprogonParametresSet, mTasks.operator [](key), &OneDeviceElectroprogonTask::onElectroprogonParametresSet);
        disconnect(this, &MainWindow::signalStartTask, mTasks.operator [](key), &OneDeviceElectroprogonTask::onStart);
        disconnect(this, &MainWindow::signalStopTask, mTasks.operator [](key), &OneDeviceElectroprogonTask::onStop);
        disconnect(mTasks.value(key), &OneDeviceElectroprogonTask::signalHasFinished, this, &MainWindow::onFinished);
        disconnect(mTasks.value(key), &OneDeviceElectroprogonTask::signalSendStatistics, this, &MainWindow::onReceiveStatistics);
        disconnect(mTasks.value(key), &OneDeviceElectroprogonTask::signalPrint, this, &MainWindow::onPrint);
        disconnect(mTasks.value(key), &OneDeviceElectroprogonTask::signalPrintLines, this, &MainWindow::onPrintLines);
        delete mTasks.value(key);
    }
    mTasks.clear();
}
